import { Wrapper, ArrayOps } from '../wrapper';
export interface Option<A> {
    /**
     * Returns the value contained in this Option.
     * This will always return undefined if this Option instance is None.
     * This method never throws.
     */
    get(): A | undefined;
    /**
     * Returns whether this Option has a defined value (i.e, it's a Some(value))
     */
    isDefined(): this is Some<A>;
    /**
     * Applies the given procedure to the option's value, if it is non empty.
     */
    forEach(fn: (a: A) => void): void;
    map<B>(fn: (a: A) => Wrapper<B> | null | undefined): Option<B>;
    map<B>(fn: (a: A) => B | null | undefined): Option<B>;
    /**
     * Maps the value contained in this Some, else returns None.
     * Depending on the map function return value, a Some could be tranformed into a None.
     */
    map<B>(fn: (a: A) => B | null | undefined | Wrapper<B>): Option<B>;
    /**
     * Maps the value contained in this Some to a new Option, else returns None.
     */
    flatMap<B>(fn: (a: A) => Option<B>): Option<B>;
    /**
     * If this Option is a Some and the predicate returns true, keep that Some.
     * In all other cases, return None.
     */
    filter(fn: (a: A) => boolean): Option<A>;
    /**
     * Applies the first function if this is a None, else applies the second function.
     * Note: Since this method creates 2 functions everytime it runs, don't use in tight loops; use isDefined() instead.
     */
    fold<B, C>(ifEmpty: () => B, ifDefined: (a: A) => C): B | C;
    /**
     * Returns this Option unless it's a None, in which case the provided alternative is returned
     */
    orElse(alternative: () => Option<A>): Option<A>;
    /**
     * Returns this Option's value if it's a Some, else return the provided alternative
     */
    getOrElse(alternative: A): A;
    /**
     * Converts this Option to an Array.
     */
    toArray(): ArrayOps<A>;
    toString(): string;
}
export interface Some<T> extends Option<T> {
    type: 'some';
    get(): T;
}
export interface None extends Option<never> {
    type: 'none';
    get(): undefined;
}
export declare type NullableValue<T> = T | Option<T> | null | undefined;
export interface OptionObject {
    /**
     * Creates an Option from a value.
     * If the value is null or undefined, it will create a None, else a Some.
     */
    <T>(value: T | null | undefined): Option<T>;
    /**
     * Returns whether the passed value is an Option (either a Some or a None).
     */
    isOption(value: any): value is Option<{}>;
    /**
     * Creates a new Option holding the tuple of all the passed values if they were all Some or non null/undefined values,
     * else returns None
     */
    all<T1, T2>(t1: NullableValue<T1>, t2: NullableValue<T2>): Option<[T1, T2]>;
    all<T1, T2, T3>(t1: NullableValue<T1>, t2: NullableValue<T2>, t3: NullableValue<T3>): Option<[T1, T2, T3]>;
    all<T1, T2, T3, T4>(t1: NullableValue<T1>, t2: NullableValue<T2>, t3: NullableValue<T3>, t4: NullableValue<T4>): Option<[T1, T2, T3, T4]>;
    all<T1, T2, T3, T4, T5>(t1: NullableValue<T1>, t2: NullableValue<T2>, t3: NullableValue<T3>, t4: NullableValue<T4>, t5: NullableValue<T5>): Option<[T1, T2, T3, T4, T5]>;
    all<T>(...ts: Array<NullableValue<T>>): Option<T[]>;
}
export declare const Option: OptionObject;
/** Creates a new Some instance using a non nullable value */
export declare function Some<T extends {}>(value: T): Some<T>;
export declare const None: None;
